import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.StreamMessage;
import javax.naming.InitialContext;

import business.Sample;

/**
 * A simple example to send a large message to queue
 */
public class LargeMsgSender {

	public static void main(final String[] args) {
		try {
			new LargeMsgSender().runExample("c:/temp/temp.pdf");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public boolean runExample(String fileName) throws Exception {
		Connection connection = null;
		InitialContext initialContext = null;

		try {
			// Step 1. Get an initial context for looking up JNDI from the
			// server #1
			initialContext = getContext(1);

			// Step 2. Look up the JMS resources from JNDI
			Queue queue = (Queue) initialContext.lookup("/queue/requestQ");
			ConnectionFactory connectionFactory = (ConnectionFactory) initialContext
					.lookup("/XAConnectionFactory");

			// Step 3. Create a JMS Connection
			connection = connectionFactory.createConnection();

			// Step 4. Create a *non-transacted* JMS Session with auto
			// acknowledgment
			Session session = connection.createSession(true,
					Session.AUTO_ACKNOWLEDGE);

			MessageConsumer queueReceiver = session.createConsumer(queue);
			queueReceiver.setMessageListener(new LargeMsgListener());
			// Step 5. Start the connection to ensure delivery occurs
			connection.start();

			// Step 6. Create a JMS MessageProducer and a MessageConsumer
			MessageProducer producer = session.createProducer(queue);

			// Step 7. Send some messages to server #1, the live server
			sendMessage(session, producer, fileName);

			Thread.sleep(2000);
			return true;
		} finally {
			// Step 8. Be sure to close our resources!
			if (connection != null) {
				connection.close();
			}
			if (initialContext != null) {
				initialContext.close();
			}
		}
	}

	/**
	 * Method to send a message to queue
	 * @param session
	 * @param producer
	 * @param fileName
	 * @throws JMSException
	 */
	private void sendMessage(Session session, MessageProducer producer,
			String fileName) throws JMSException {
		final int numMessages = 1;
		StreamMessage message = null;

		for (int i = 0; i < numMessages; i++) {
			message = session.createStreamMessage();

			// Attach a java object and a file to the message
			attachObject(message, "ObjectMessage -> 1");
			attachFile(message, fileName);

			// send message to queue
			producer.send(message);
			System.out.println("Large message sent to queue ");
		}
		session.commit();
	}

	/**
	 * Method to attach an object to JMS message
	 * @param message
	 * @param name
	 */
	private void attachObject(StreamMessage message, String name) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		Sample sample = new Sample(name);
		try {
			ObjectOutputStream oos = new ObjectOutputStream(baos);
			oos.writeObject(sample);
			oos.flush();
			oos.close();
			baos.close();
			byte[] data = baos.toByteArray();

			message.writeBytes(data);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (JMSException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Method to attach a file to JMS message
	 * @param message
	 * @param fileName
	 */
	private void attachFile(StreamMessage message, String fileName) {
		FileInputStream fileInputStream = null;

		try {
			fileInputStream = new FileInputStream(fileName);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		BufferedInputStream bufferedInput = new BufferedInputStream(
				fileInputStream);
		try {
			message.setObjectProperty("JMS_HQ_InputStream", bufferedInput);
			message.setStringProperty("FileName", fileName.substring(fileName
					.lastIndexOf('/') + 1));
		} catch (JMSException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Listener class to consume JMS messages
	 */
	public class LargeMsgListener implements MessageListener {
		public LargeMsgListener() {
		}

		public void onMessage(Message message) {
			try {
				System.out.println("In Listener OnMessage");
				StreamMessage messageReceived = (StreamMessage) message;

				detachObject(messageReceived);
				detachFile(messageReceived);

				message.acknowledge();
			} catch (Throwable e) {
				System.out.println("onMessage Thr-> " + e.getMessage());
			}
		}

		/**
		 * Method to extract the obect contents from the message and display the same
		 * @param message
		 */
		private void detachObject(StreamMessage message) {
			byte[] data = new byte[256];
			try {
				//Read object details from message as byte[]
				int numberOfBytes = message.readBytes(data);
				
				//Display object contents in console
				printObjectDetails(data);
				
			} catch (Throwable e) {
				System.out.println("detachObject() -> " + e.getMessage());
			}
		}

		/**
		 * Method to extract the file attachment from the message and store in temp folder
		 * @param messageReceived
		 */
		private void detachFile(StreamMessage messageReceived) {
			try {

				String fileName = messageReceived.getStringProperty("FileName");
				File outputFile = new File("D:/temp/" + fileName);
				FileOutputStream fileOutputStream = new FileOutputStream(
						outputFile);
				BufferedOutputStream bufferedOutput = new BufferedOutputStream(
						fileOutputStream);

				messageReceived.setObjectProperty("JMS_HQ_SaveStream",
						bufferedOutput);
				fileOutputStream.close();
				System.out
						.println("File streamed to disk. Size of received file on disk is "
								+ outputFile.length());
			} catch (Exception e) {
				System.out.println("readFile -> " + e.getMessage());
			}
		}
		
		/**
		 * Method to print the object details
		 * @param data
		 * @throws IOException
		 * @throws ClassNotFoundException
		 */
		private void printObjectDetails(byte[] data) throws IOException,
				ClassNotFoundException {
			Sample sample = (Sample) new java.io.ObjectInputStream(
					new java.io.ByteArrayInputStream(data)).readObject();
			System.out.println("Object content: " + sample.getName());
		}
	}

	protected InitialContext getContext(final int serverId) throws Exception {
		Properties props = new Properties();

		HashMap map = new HashMap();
		map.put("java.naming.factory.initial",
				"org.jnp.interfaces.NamingContextFactory");
		map.put("java.naming.provider.url", "localhost:1099");
		map.put("java.naming.factory.url.pkgs",
				"org.jboss.naming:org.jnp.interfaces");
		props.putAll(map);
		return new InitialContext(props);
	}
}